#!/bin/sh

LUAROCKS_SYSCONFDIR='/etc/luarocks' exec '/usr/bin/lua5.5' -e 'package.path="/usr/share/lua/5.5/?.lua;/build/lua-luarepl/pkg/lua-luarepl/usr/share/lua/5.5/?/init.lua;/build/.luarocks/share/lua/5.5/?.lua;/build/.luarocks/share/lua/5.5/?/init.lua;"..package.path;package.cpath="/build/lua-luarepl/pkg/lua-luarepl/usr/lib/lua/5.5/?.so;/build/.luarocks/lib/lua/5.5/?.so;"..package.cpath;local k,l,_=pcall(require,"luarocks.loader") _=k and l.add_context("luarepl","0.10-1")' '/build/lua-luarepl/pkg/lua-luarepl/usr/lib/luarocks/rocks-5.5/luarepl/0.10-1/bin/rep.lua' "$@"
